// BSRouteTab.cpp : implementation file
//

#include "stdafx.h"
#include "SensorSim.h"
#include "BSRouteTab.h"
#include "SensorSimCore.h"
#include "SensorSimDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSensorSimCore*  pSensorSim;
extern CSensorSimDlg*  pSensorSimDlg;

/////////////////////////////////////////////////////////////////////////////
// CBSRouteTab dialog


CBSRouteTab::CBSRouteTab(CWnd* pParent /*=NULL*/)
	: CDialog(CBSRouteTab::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBSRouteTab)
	m_bsCountTotal = _T("0");
	m_bsCountUsed = _T("0");
	m_neighDist = _T("0.5");
	m_routeStategy = 1;
	m_activeBSSelect = 0;
	m_bsPlacement = 2;
	//}}AFX_DATA_INIT
}


void CBSRouteTab::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBSRouteTab)
	DDX_Text(pDX, IDC_BSCOUNT_TOTAL, m_bsCountTotal);
	DDX_Text(pDX, IDC_BSCOUNT_USED, m_bsCountUsed);
	DDX_Text(pDX, IDC_NEIGH_DIST, m_neighDist);
	DDX_Radio(pDX, IDC_ROUTESTRAT_GREEDY, m_routeStategy);
	DDX_Radio(pDX, IDC_ACTIVEBS_NEAREST, m_activeBSSelect);
	DDX_Radio(pDX, IDC_BS_PLACEMENT, m_bsPlacement);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBSRouteTab, CDialog)
	//{{AFX_MSG_MAP(CBSRouteTab)
	ON_BN_CLICKED(IDC_MMD, OnMmd)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int CBSRouteTab::GetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;
    
    UpdateData(TRUE);

	pBasicSettings->bsConfig.bsCount = atol(m_bsCountTotal);
    pBasicSettings->bsConfig.bsActive = atol(m_bsCountUsed);
    pBasicSettings->bsConfig.bsPlacement = m_bsPlacement;
    pBasicSettings->mmdConfig.routingStrategy = m_routeStategy;
	pBasicSettings->mmdConfig.nextNeighDistance = (float) atof(m_neighDist);

    return status;
}


int CBSRouteTab::SetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;
    
	m_bsCountTotal.Format("%d", pBasicSettings->bsConfig.bsCount);
	m_bsCountUsed.Format("%d", pBasicSettings->bsConfig.bsActive);
    m_routeStategy = pBasicSettings->mmdConfig.routingStrategy;
	m_neighDist.Format("%.2f", pBasicSettings->mmdConfig.nextNeighDistance);
    m_bsPlacement = pBasicSettings->bsConfig.bsPlacement;

	UpdateData(FALSE);

    return status;
}

/////////////////////////////////////////////////////////////////////////////
// CBSRouteTab message handlers

void CBSRouteTab::OnMmd() 
{
    int             status = STAT_OK;
    BASIC_INIT_DATA basicSettings;
    MMD_TEST_RESULTS mmdResults;

    // GET SETTINGS
    status = pSensorSimDlg->GetBasicSettings(&basicSettings);

    // INIT RANDOM GENERATOR
    pSensorSim->m_sensorEnv.InitRandomGenerator(&basicSettings);

    // DEPLOY SENSORS AND BASE STATIONS
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.DeploySensors(&basicSettings);
    }

    // SELECT ACTIVE BS FOR EACH SENSOR AND CREATE ROUTING TABLES
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.MMD_CreateRoutingPaths(&basicSettings);
    }

    // TRY ROUTE MESSAGE FROM SENSOR TO EACH BS USING SELECT ROUTING STRATEGY
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.MMD_Test(&basicSettings, &mmdResults);
    }
}

