// NeighLeashesTab.cpp : implementation file
//

#include "stdafx.h"
#include "SensorSim.h"
#include "NeighLeashesTab.h"
#include "SensorSimCore.h"

extern CSensorSimCore*  pSensorSim;
extern CSensorSimDlg*  pSensorSimDlg;
   
// CNeighLeashesTab

IMPLEMENT_DYNCREATE(CNeighLeashesTab, CDialog)

CNeighLeashesTab::CNeighLeashesTab()
	: CDialog(CNeighLeashesTab::IDD)
    , m_dropProb(_T("1"))
    , m_injectProb(_T("0"))
    , m_changeDestProb(_T("0"))
    , m_neighTransmitCheckProb(_T("0.5"))
    , m_fractComprNodes(_T("0.1"))
{
    m_transmitCheck = 0;
}

CNeighLeashesTab::~CNeighLeashesTab()
{
}

void CNeighLeashesTab::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Text(pDX, IDC_DROPPROB, m_dropProb);
    DDX_Text(pDX, IDC_INJECTPROB, m_injectProb);
    DDX_Text(pDX, IDC_CHANGEDESTPROB, m_changeDestProb);
    DDX_Text(pDX, IDC_NEIGHCHECKPROB, m_neighTransmitCheckProb);
    DDX_Radio(pDX, IDC_TRANSMITCHECK_SENDERONLY, m_transmitCheck);

    DDX_Text(pDX, IDC_FRACT_COMPRNODES, m_fractComprNodes);
}

BEGIN_MESSAGE_MAP(CNeighLeashesTab, CDialog)
    ON_BN_CLICKED(IDC_TEST_ROUTING, OnBnClickedTestRouting)
	ON_BN_CLICKED(IDC_RUN_GA_ROUTE, &CNeighLeashesTab::OnBnClickedRunGaRoute)
END_MESSAGE_MAP()


int CNeighLeashesTab::GetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;
    
    UpdateData(TRUE);

	pBasicSettings->neighLeash.dropMessageProb = atof(m_dropProb);
	pBasicSettings->neighLeash.injectMessageProb = atof(m_injectProb);
	pBasicSettings->neighLeash.changeMessageDestProb = atof(m_changeDestProb);
	pBasicSettings->neighLeash.neighTransmitCheckProb = atof(m_neighTransmitCheckProb);
	pBasicSettings->neighLeash.fractComprNodes = atof(m_fractComprNodes);

    switch (m_transmitCheck) {
        case 0: pBasicSettings->neighLeash.transmitCheckFlags |= FLAG_TRANSMITCHECK_SENDERONLY; break;
        case 1: pBasicSettings->neighLeash.transmitCheckFlags |= FLAG_TRANSMITCHECK_NEIGHPROB; break;
        case 2: pBasicSettings->neighLeash.transmitCheckFlags |= FLAG_TRANSMITCHECK_ALLNEIGHS; break;
    }

    return status;
}


int CNeighLeashesTab::SetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;
   
	m_dropProb.Format("%.2f", pBasicSettings->neighLeash.dropMessageProb);
	m_injectProb.Format("%.2f", pBasicSettings->neighLeash.injectMessageProb);
	m_changeDestProb.Format("%.2f", pBasicSettings->neighLeash.changeMessageDestProb);
	m_neighTransmitCheckProb.Format("%.2f", pBasicSettings->neighLeash.neighTransmitCheckProb);
	m_fractComprNodes.Format("%.2f", pBasicSettings->neighLeash.fractComprNodes);

    if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_SENDERONLY) m_transmitCheck = 0;
    if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_NEIGHPROB) m_transmitCheck = 1;
    if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_ALLNEIGHS) m_transmitCheck = 2;

	UpdateData(FALSE);

    return status;
}

// CNeighLeashesTab diagnostics
#ifdef _DEBUG
void CNeighLeashesTab::AssertValid() const
{
	CDialog::AssertValid();
}

#ifndef _WIN32_WCE
void CNeighLeashesTab::Dump(CDumpContext& dc) const
{
	CDialog::Dump(dc);
}
#endif
#endif //_DEBUG


// CNeighLeashesTab message handlers

void CNeighLeashesTab::OnBnClickedTestRouting()
{
    int                     status = STAT_OK;
    BASIC_INIT_DATA         basicSettings;

    LOG_INSERTSTRING("TestRouting start");

    // GET SETTINGS AND INIT RANDOM GENERATOR
    if (status == STAT_OK) {
        status = pSensorSimDlg->GetBasicSettings(&basicSettings);
        pSensorSim->m_sensorEnv.InitRandomGenerator(&basicSettings);
    }
    
    // DEPLOY SENSORS AND BASE STATIONS 
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.DeploySensors(&basicSettings);
        pSensorSim->m_sensorEnv.PrintBasicDeployInfo();
        LOG_INSERTSTRING("After deployment");
    }
    
		
	//JK
	if (status == STAT_OK) {
		if (basicSettings.mmdConfig.routingStrategy==MMD_ROUTESTRAT_IGF) {
			status = pSensorSim->m_sensorEnv.Routing_Test(&basicSettings);
		}
	}

    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.NLeash_Test(&basicSettings, TRUE);    
    }
}


//JK
void CNeighLeashesTab::OnBnClickedRunGaRoute()
{
	BASIC_INIT_DATA  basicSettings;

	// GET SETTINGS AND INIT RANDOM GENERATOR
    pSensorSimDlg->GetBasicSettings(&basicSettings);
    pSensorSim->m_sensorEnv.InitRandomGenerator(&basicSettings);

	CSensorEnv::RunGARouteEvol(&basicSettings);
}
//JK
int CNeighLeashesTab::GetGARouteData(GA_ROUTE* pGARoute) {
	int status = STAT_OK;

	return status;
}
