#include "Proxy.h"
#include "SensorSimCore.h"
#include "CommonFnc.h"

extern CSensorSimCore* pSensorSim;

int CProxy::m_numSocketConnections = 0;

CProxy::CProxy(void) {
	m_pSocketServer = NULL;
	m_pProxyThread = NULL;	
}

CProxy::~CProxy(void) {
	if (m_pProxyThread != NULL) {
		TerminateThread(m_pProxyThread, 0);
		m_pProxyThread = NULL;
	}

	if (m_pSocketServer != NULL) {
		delete m_pSocketServer;
		m_pSocketServer = NULL;
	}
}


UINT CProxy::StartProxy(void* a) {
    SocketServer* pSocketServer = (SocketServer*) a; 

    LOG_INSERTSTRING("Proxy started");

    while (1) {
        try {
            Socket* s=pSocketServer->Accept();

            LOG_INSERTSTRING("\nNew connection detected and waiting for service");

            // TERMINATE IF REQUIRED
            if (pSensorSim->m_bStopThread) break;

            while (CProxy::m_numSocketConnections > 0) {
                // WAIT UNTIL PREVIOUS CONNECTION FINISH
            }

            // INCREASE NUMBER OF SERVED ACTIVE CONNECTIONS
            CProxy::m_numSocketConnections++;
            AfxBeginThread(CProxy::Answer, (void*) s, THREAD_PRIORITY_NORMAL); 
        }
        catch(...) {
            // TERMINATE IF REQUIRED (POSSIBLE REASON FOR EXCEPTION)
            if (pSensorSim->m_bStopThread) break;
            else LOG_INSERTSTRING("Exception in pSocketServer->Accept");        
        }
    }

    return 0; 
}

void CProxy::RunProxy(int port) {
    m_pSocketServer = new SocketServer(port, 1);   

    CString message;
    message.Format("Starting proxy on port %d", port);
	LOG_INSERTSTRING(message);

    m_pProxyThread = AfxBeginThread(CProxy::StartProxy, (void*) m_pSocketServer, THREAD_PRIORITY_NORMAL); 
}

int CProxy::SocketSendLine(SocketClient* pSocket, CString line, CString* pResponse, CString* pStatus) {
	int		status = STAT_OK;
	int		pos = 0;
	int		pos2 = 0;

	pSocket->SendLine((LPCTSTR) line);

    string l = pSocket->ReceiveResponse(SCSAT_SOCKET_ENDSEQ, SCSAT_SOCKET_TIMEOUT);
	*pResponse = l.c_str();
	
	// SEARCH FOR RESULT OF QUERY (command#status#...)
	if ((pos = pResponse->Find("#")) != -1) {
		if ((pos2 = pResponse->Find("#", pos + 1)) != -1) {
			// SAVE status
			if (pStatus != NULL) *pStatus = pResponse->Mid(pos + 1, pos2 - pos - 1);
			
			// REMOVE command#status
			CString temp = pResponse->Mid(pos2); 

			// REMOVE ENDING @@ - SCSAT_SOCKET_ENDSEQ
			*pResponse = temp.Left(temp.Find(SCSAT_SOCKET_ENDSEQ)); 
		}
		else status = STAT_DATA_INCORRECT_LENGTH;
	}
	else status = STAT_DATA_INCORRECT_LENGTH;

	return status;
}

UINT CProxy::Answer(void* a) {
    Socket* s = (Socket*) a;
    int         status = STAT_OK;
    CString     value;
    CString     command;
    CString     response;
    CString     message;
    int         pos = 0;
    int         pos2 = 0;

    while (1) {
        Sleep(100);
   
        // TERMINATE
        if (pSensorSim->m_bStopThread) return 0; 
   
        std::string r = s->ReceiveLine();
        value = r.c_str();
        
        if (value != "") { 
            message.Format("\n<< %s", value);
			LOG_INSERTSTRING(message);

            pos = value.Find(PROXY_SEPARATOR);
            command = value.Left(pos);
            pos++;
            
            BOOL    bRecognizedCommand = FALSE;
            
            // GET RUNNING INSTANCES
            if (command.CompareNoCase(PROXY_GETINSTANCES) == 0) {
                bRecognizedCommand = TRUE;

				// OPEN FILE WITH INSTANCES LOGS
				int		retStatus = STAT_OK;
				lcs		instances;
				CString usedLog = S3_INSTANCES_LOGS_C;
				// TRY TO LOAD FROM FIRST ACCESSIBLE LOG FILE
				if (CCommonFnc::File_GetIniSections(S3_INSTANCES_LOGS_C, &instances) != STAT_OK) {
				    usedLog = S3_INSTANCES_LOGS_D;
				    if (CCommonFnc::File_GetIniSections(S3_INSTANCES_LOGS_D, &instances) != STAT_OK) {
                        usedLog = S3_INSTANCES_LOGS_E;
                        retStatus = CCommonFnc::File_GetIniSections(S3_INSTANCES_LOGS_E, &instances);
                    }
                }
				
				if (retStatus == STAT_OK) {
					response.Format("%s#OK#", PROXY_GETINSTANCES);        
					
					// PASS OVER ALL SECTIONS
					lcs::iterator	iter;
					CString			instanceString;
				    char			iniValue[MAX_INI_VALUE_CHAR];
					for (iter = instances.begin(); iter != instances.end(); iter++) {
						instanceString = *iter;
						instanceString += PROXY_VALUE_SEPARATOR;
/*
						// CHECK FOR INSTANCE LIVENESS
						CTime     lastReset = CTime::GetCurrentTime();
						CTime     actualTime = CTime::GetCurrentTime();
						CTimeSpan ts(0, 6, 0, 0 ); 
						CTimeSpan ts2(0, 0, 5, 0 ); 

						actualTime = CTime::GetCurrentTime();
						if ((actualTime - ts) > lastReset) {
						}
						S3_INSTANCES_TIME_DEAD
/**/						
						GetPrivateProfileString(*iter, "STATUS", "", iniValue, MAX_INI_VALUE_CHAR, usedLog);
						instanceString += iniValue;
						instanceString += PROXY_VALUE_SEPARATOR;
						GetPrivateProfileString(*iter, "DATE", "", iniValue, MAX_INI_VALUE_CHAR, usedLog);
						instanceString += iniValue;

						response += instanceString;        
						response += PROXY_SEPARATOR;
					}
				}
				else {
					response.Format("%s#FAIL#%d#", PROXY_GETINSTANCES, retStatus);        
				}

				response += SCSAT_SOCKET_ENDSEQ;
				std::string  cmd((LPCTSTR) response);
                s->SendLine(cmd);
                
                break;
            }

            // STOP
            if (command.CompareNoCase(PROXY_DISCONNECT) == 0) {
                bRecognizedCommand = TRUE;

                response.Format("%s#OK#", PROXY_DISCONNECT);        
                response += SCSAT_SOCKET_ENDSEQ;
                
                std::string  cmd((LPCTSTR) response);
                s->SendLine(cmd);
                
                break;
            }
            
            // GET FILE CONTENT
            if (command.CompareNoCase(PROXY_GETTEXTFILECONTENT) == 0) {
                bRecognizedCommand = TRUE;

                // GET PARAMS
                CString filePath;
                
                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    filePath = value.Mid(pos, pos2 - pos);    
                    pos = pos2 + 1;
                }
                
                if (filePath != "") { 
                    // LOCATE RESULTS FILE
                    CString targetFile;
                    CString results;    
                    CFile   file;
                    targetFile.Format("%s\\%s", pSensorSim->m_basePath, filePath);
                    // READ CONTENT OF THE FILE
                    if (file.Open(targetFile, CFile::modeRead)) {
                        int len = file.GetLength() + 100;
                        char*   buffer = new char[len];
                        memset(buffer, 0, len);
                        file.Read(buffer, len);     
                        results = buffer;
                        delete[] buffer;

                        response.Format("%s#OK#%s#", PROXY_GETTEXTFILECONTENT, results); 
                    }
                    else {
                        response.Format("%s#FAIL#%d#FailToOpenTargetFile#", PROXY_GETTEXTFILECONTENT, status);        
                    }
                }
                else {
                    response.Format("%s#FAIL#%d#NotEnoughtParameters#", PROXY_GETTEXTFILECONTENT, status);        
                }
            }
            
			
            // RUN APPLICATION
            if (command.CompareNoCase(PROXY_RUNAPPLICATION) == 0) {
                bRecognizedCommand = TRUE;

                // GET PARAMS
                CString filePath;
				CString params;

                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    filePath = value.Mid(pos, pos2 - pos);    
                    pos = pos2 + 1;
                }
                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    params = value.Mid(pos, pos2 - pos);    
                    pos = pos2 + 1;
                }
                
                if (filePath != "") { 
                    // LOCATE RESULTS FILE
                    CString cmdLine;
                    cmdLine.Format("\"%s\\%s\" %s", pSensorSim->m_basePath, filePath, params);
                
					WinExec(cmdLine, 0);

                    response.Format("%s#OK#", PROXY_RUNAPPLICATION); 
                }
                else {
                    response.Format("%s#FAIL#%d#NotEnoughtParameters#", PROXY_RUNAPPLICATION, status);        
                }
            }

            // COPY FILE
            if (command.CompareNoCase(PROXY_COPYFILE) == 0) {
                bRecognizedCommand = TRUE;

                // GET PARAMS
                CString origFilePath;
				CString targetFilePath;
                CString p1;
				CString p2;

                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    p1 = value.Mid(pos, pos2 - pos);    
                    pos = pos2 + 1;
                }
                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    p2 = value.Mid(pos, pos2 - pos);    
                    pos = pos2 + 1;
                }
                
                if (p1 != "" && p2 != "") { 
					origFilePath.Format("%s\\%s", pSensorSim->m_basePath, p1);
					targetFilePath.Format("%s\\%s", pSensorSim->m_basePath, p2);

                    // CREATE DIRECTORY
                    CString dirPath;
                    CCommonFnc::Path_ExtractDirPath(targetFilePath, &dirPath);
                    CCommonFnc::Dir_Create(dirPath);

					CopyFile(origFilePath, targetFilePath, FALSE);

                    response.Format("%s#OK#", PROXY_COPYFILE); 
                }
                else {
                    response.Format("%s#FAIL#%d#NotEnoughtParameters#", PROXY_COPYFILE, status);        
                }
            }


			// SET FILE CONTENT
            if (command.CompareNoCase(PROXY_SETFILECONTENT) == 0) {
                bRecognizedCommand = TRUE;

                // GET PARAMS
                CString filePath;
                int dataLength;
                
                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    filePath = value.Mid(pos, pos2 - pos);    
                    pos = pos2 + 1;
                }
                value += PROXY_SEPARATOR;
                if ((pos2 = value.Find(PROXY_SEPARATOR, pos)) != -1) {
                    dataLength = atoi(value.Mid(pos, pos2 - pos));    
                    pos = pos2 + 1;
                }
                
                if (filePath != "" && dataLength > 0) { 
                    // LOCATE RESULTS FILE
                    CString targetFile;
                    CString results;    
                    CFile   file;
                    targetFile.Format("%s\\%s", pSensorSim->m_basePath, filePath);

                    // CREATE DIRECTORY
                    CString dirPath;
                    CCommonFnc::Path_ExtractDirPath(targetFile, &dirPath);
                    CCommonFnc::Dir_Create(dirPath);
                    
                    // WRITE CONTENT OF THE FILE
                    if (s->ReceiveBytesToFile((LPCTSTR) targetFile, dataLength + 1, SCSAT_SOCKET_TIMEOUT) == 0) {
                        response.Format("%s#OK#%s#", PROXY_SETFILECONTENT, results); 

                    }
                    else {
                        response.Format("%s#FAIL#%d#FailToReceiveToTargetFile#", PROXY_SETFILECONTENT, status);        
                    }
                
                    response.Format("%s#OK#%s#", PROXY_SETFILECONTENT, results); 
                }
                else {
                    response.Format("%s#FAIL#%d#NotEnoughtParameters#", PROXY_SETFILECONTENT, status);        
                }
            }
            
            if (!bRecognizedCommand) {
                response.Format("##FAIL#UnrecognizedCommand#");        
            }
            
            response += SCSAT_SOCKET_ENDSEQ;
            std::string  cmd(response);
            s->SendLine(cmd);
            
            message.Format(">> %s\n", response);
			LOG_INSERTSTRING(message);
        }
    }

    // DECREASE NUMBER OF ACTIVE CONNECTIONS
    CProxy::m_numSocketConnections -= 1;

    delete s;


    return 0;
}
