#ifndef SENSORENV_H
#define SENSORENV_H

#include "SSGlobals.h"
#include "crypto/RndGen.h"
#include "Action.h"
#include "GAS_Genome.h"

class CSensorEnv {
public:
	DEPLOYMENT_INFO*	m_pActualDeployment;	// sensor structures for actual deployment
	DEPLOYMENT_INFO*	m_deploymentList[MAX_DEPLOYMENTS];

	DATA_LIST           m_ssKeysList;
    int                 m_ssNextKeyID;
    DATA_LIST           m_ssMessagesList;
    DATA_LIST           m_ssAttackerKeysList;
    DATA_LIST           m_ssAttackerMessagesList;

    CRndGen             m_rndGen;

    GAPopulation*       m_pLoadedPop;
    int                 m_lastLoadedGenom;
    
    BASIC_INIT_DATA     m_basicSettings;
    
    
    DWORD               bestNC[1000];
    
private:
    DATA_LIST           m_ssSENSITIVE_OBJECT_PtrList;
    DATA_LIST           m_ssSENSITIVE_OBJECT_PtrPtrList;
    DATA_LIST           m_ssSENSOR_NODE_PtrList;
    DATA_LIST           m_ssSENSOR_NODE_PtrPtrList;
public:
    CSensorEnv();
    ~CSensorEnv();

    int DeploySensors(BASIC_INIT_DATA* pBasicSettings);
    int TestDeployIntegrity();

    static int RemoveDeployedNodes(DWORD flags);
    static int DeployBlackNodesByPattern(BASIC_INIT_DATA* pBasicSettings, NODES_DEPLOY_PATTERN* pDeployPattern);
    static int DeployWhiteNodesByPattern(BASIC_INIT_DATA* pBasicSettings, NODES_DEPLOY_PATTERN* pDeployPattern);
    static int DeployNodesByPattern(BASIC_INIT_DATA* pBasicSettings, NODES_DEPLOY_PATTERN* pDeployPattern, DWORD flags);
     
    void Clear();
    void ClearLinkKeysStatus();
    void ResetNetwork();
    void Release();

    static int SaveConfigScript(CString filePath, BASIC_INIT_DATA* pBasicSettings);
    static int LoadConfigScript(CString filePath, BASIC_INIT_DATA* pBasicSettings);

    int ExecuteTestsByScript(CString filePath);
    int ExecuteTestsBySettings(BASIC_INIT_DATA* pBasicSettings);

    void CalcWhiteNeighStats(DWORD* pMax, DWORD* pMin, float* pAvg, float* pAvgWhiteBeforeBlack = NULL, float* pAvgDistNeigh = NULL);
    void CalcBlackNeighStats(DWORD* pMax, DWORD* pMin, float* pAvg);
    void CalcBlackAreaStats(BASIC_INIT_DATA* pBasicSettings, float* pBlackArea);

    // 
    // KEY INFECTION FUNCTION
    //
    int PlaintextMaxScream(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc = FALSE); 
    int BasicWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc = FALSE);
    int MutualWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc = FALSE);
    int RossAmplifMaxScream(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc = FALSE);
    int RossAmplifWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc = FALSE);
    int RossAmplifMutualWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc = FALSE);
    int DanAmplifWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc = FALSE);
    int DanAmplifMutualWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc = FALSE);
    int RossAmplif(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr);
    static int DanAmplif(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr);
    int PullAmplifEGScheme(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat);
    int PushAmplifEGScheme(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat);

    // 
    // KEY PREDISTRIBUTION FUNCTION
    //
    int EGScheme(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pMissing, BOOL bMarkMissingAsCompr = FALSE); // estimate success of EG key predistribution
    int GetKeysFromNodes(SENSOR_NODE** pNodes, DWORD numNodes, KEY_ID* pKeysList, DWORD* pKeysListSize);  // extract keys from given nodes
    int GetKeysFromNodes(SENSOR_NODE** pNodes, DWORD numNodes, BYTE* pKeysList);    // set 1 to pKeysList offset corresponding to extracted key ID
    int GetKeysFromNodes2D(SENSOR_NODE** pNodes, DWORD numNodes, BYTE* pKeysList, KEY_2DID* pKeysList2D); 
	int GetKeysFromRandomNodes(DWORD numNodes, KEY_ID* pKeysList, DWORD* pKeysListSize);  // selects randomly numNodes nodes from white sensor list
    int GetKeysFromRandomNodesBitMask(DWORD numNodes, KEYS_MASK* pKeysMask, DWORD* pKeysMaskSize);  // selects randomly numNodes nodes from white sensor list
    static int ComputeSharedKeysSet(SENSOR_NODE* pSensorNode1, SENSOR_NODE* pSensorNode2, KEY_ID* pSharedKeys, DWORD* pNumSharedKeys);            
    int ComputeSharedKeysSetBitMask(SENSOR_NODE* pSensorNode1, SENSOR_NODE* pSensorNode2, KEYS_MASK* pSharedKeysMask, DWORD* pKeysMaskSize);            
    int GetNumberKeysBitMask(KEYS_MASK* pKeysMask, DWORD keysMaskSize, DWORD* pNumKeys);

    //
    // MULTIPATH MESSAGE DELIVERY
    //
    int MMD_CreateRoutingPaths(BASIC_INIT_DATA* pBasicSettings, double* pAvgHopLen = NULL);
    int MMD_EstablishBSPaths(BASIC_INIT_DATA* pBasicSettings, DWORD* pathsFound = NULL, DWORD* pathsNotFound = NULL);
    int MMD_Test(BASIC_INIT_DATA* pBasicSettings, MMD_TEST_RESULTS* pResults);


    // FORWARD ONION ENCRYPTION
    int FOE_RecordMessagesPath(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, INT_DATA_BLOB* pCountHopsToBS = NULL, INT_DATA_BLOB* pCountHopsToCapture = NULL);
    int FOE_MessageComputeAppliedKeys(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList);
    int FOE_PrepareMessagesPath(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, DWORD* pathsFound, DWORD* pathsNotFound, INT_DATA_BLOB* pCountHopsToBS = NULL, INT_DATA_BLOB* pCountHopsToCapture = NULL);
    int FOE_Test(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, DWORD numComprNodes, FOE_TEST_RESULTS* pResults);
    static int FOE_ComputeSharedKeysSet(LIST_NODE* pNodesOnPath, SENSOR_NODE* pTargetNode, KEY_ID* pKeysList, DWORD* pKeysListSize);
    int FOE_ComputeSharedKeysSetBitMask(LIST_NODE* pNodesOnPath, SENSOR_NODE* pTargetNode, KEYS_MASK* pKeysMask, DWORD* pKeysMaskSize);
    static int PEGA_ExecuteOverPartiallySecureLinks(BASIC_INIT_DATA* pBasicSettings);
    
    // NEIGHBOUR LEASHES
    int NLeash_Test(BASIC_INIT_DATA* pBasicSettings, BOOL verbose = TRUE, CString* pResultString = NULL);
    int NLeash_MaliciousRouteTest(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, DWORD numComprNodes, NLEASH_TEST_RESULTS* pResults);
    
	int GetSharedKeysStatus(NEIGH_LIST_NODE* pListNode, DWORD* numSharedKeys, DWORD* numSecuredKeys, BYTE* pComprKeysStatusList, KEY_2DID* pComprKeysStatusList2D=NULL);
    int GetSharedIntermKeysStatus(NEIGH_LIST_NODE* pListNode, DWORD* numSharedIntermKeys, DWORD* numSecuredIntermKeys, BYTE* pComprKeysStatusList);
	int DistributeKeyStatusRandomly(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD distribFlags = 0, BOOL bForceNewKeyDistribution = 1);
//    int DistributeKeyStatusByComprList(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, KEY_ID* pComprKeysList, DWORD comprKeysListSize);
    int DistributeKeyStatusByComprList(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BYTE* pComprKeysStatusList, BOOL bMarkMissingAsCompr);
	int DistributeKeyStatusByComprList2D(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BYTE* pComprKeysStatusList,KEY_2DID* pComprKeysStatusList2D, SENSOR_NODE** pComprNodeList=NULL, DWORD numCaptured=0, BOOL bMarkMissingAsCompr=FALSE);
	int SetLinkKeyStatusIntermediate(BASIC_INIT_DATA* pBasicSettings, NEIGH_LIST_NODE* pListNode, DWORD numSharedKeys, DWORD numSharedIntermKeys, DWORD numSecuredKeys, BOOL bMarkMissingAsCompr);
	int SetLinkKeyStatus(BASIC_INIT_DATA* pBasicSettings, NEIGH_LIST_NODE* pListNode, DWORD numSharedKeys, DWORD numSecuredKeys, BOOL bMarkMissingAsCompr);
	BOOL IsNodeInList(SENSOR_NODE* pSensorNode, SENSOR_NODE** pList,DWORD listLength);

    int DistributeEqualKeyStatusRandomly(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD distribFlags = 0);
	int ComputeNumberOfComprKeysByBlack();
    int SetMostComprAsInactiveNeigh(BASIC_INIT_DATA* pBasicSettings);

    int InitRandomGenerator(BASIC_INIT_DATA* pBasicSettings);

    int SaveWhitePositions(CString fileName);
    int SaveBlackPositions(CString fileName);
    int SaveSecureKeyDistrib(CString fileName);
    int SaveCompromissedByBlack(CString fileName);
    int SaveCompromissedByDistance(BASIC_INIT_DATA* pBasicSettings, CString fileName);
    int SaveCompromissedByNodeDensity(BASIC_INIT_DATA* pBasicSettings, CString fileName);
    int SaveWhiteTriangleProperties(CString fileName);
    
    static int SaveInstanceLog(CString message);
/*
    static int CreateTestHeader(CString testName, BASIC_INIT_DATA* pBasicSettings, CString basicPath, CString columns = "");
    static int FormatSettingsString(BASIC_INIT_DATA* pBasicSettings, CString* pString);
/**/
    
    // SCRIPT STRATEGY SUPPORT
    int SS_ExecuteStrategies(lss* pStrategies);
    int SS_ExecuteStrategy(SCRIPT_STRATEGY* pStrategy);
    int SS_ExecutePUSHStrategy(SCRIPT_STRATEGY* pStrategy);
    int SS_AllocateObjectsCrossPointers(SCRIPT_STRATEGY* pStrategy);
    int SS_SendMessage(SENSOR_NODE* pSensor1, SENSOR_NODE* pSensor2, SENSITIVE_OBJECT* pMessage);
    int SS_ProcessMessages();
    int SS_ProcessMessage(SENSOR_NODE* pSensor, SENSITIVE_OBJECT* pMessage);
    int SS_ResendMessage(SENSOR_NODE* pSender, SENSOR_NODE* pTarget, SENSITIVE_OBJECT* pOrigMessage, SENSITIVE_OBJECT* pNewKey, SENSITIVE_OBJECT** ppNewMessage);
    int SS_ForwardMessage(SENSOR_NODE* pSender, SENSOR_NODE* pTarget, SENSITIVE_OBJECT* pMessage);
    int SS_CombineKeys(SENSOR_NODE* pSensor1, SENSOR_NODE* pSensor2, DATA_LIST* pKeys, SENSITIVE_OBJECT** ppKeyComposite);
    int SS_CombineKeys2(SENSITIVE_OBJECT* pKey1, SENSITIVE_OBJECT* pKey2, SENSITIVE_OBJECT** ppKeyComposite);
    int SS_GetSharedKeys(SENSOR_NODE* pSensor1, SENSOR_NODE* pSensor2, DATA_LIST* pKeys);

    int SS_AttackerInterceptMessage(SENSITIVE_OBJECT* pMessage);
    int SS_AttackerInterceptKey(SENSITIVE_OBJECT* pMessage);
    int SS_AttackerHaveAllKeys(SENSITIVE_OBJECT* pMessage);

    int SS_NewKeyObject(DWORD typeFlags, SENSITIVE_OBJECT** ppKey);
    int SS_NewMessageObject(DATA_LIST* pComponents, DATA_LIST* pContent, SENSITIVE_OBJECT** ppMessage);
    int SS_NewMessageObject(SENSITIVE_OBJECT* pKey, DATA_LIST* pContent, SENSITIVE_OBJECT** ppMessage);
    int SS_NewMessageObject(SENSITIVE_OBJECT* pKey, SENSITIVE_OBJECT* pContent, SENSITIVE_OBJECT** ppMessage);
	int	SS_DuplicateContent(DATA_LIST* pOrigContent, DATA_LIST** ppDuplicateContent);

    void SS_AddPointerToList(SENSITIVE_OBJECT* pPtr);
    void SS_AddPointerToList(SENSITIVE_OBJECT** pPtr);
    void SS_AddPointerToList(SENSOR_NODE* pPtr);
    void SS_AddPointerToList(SENSOR_NODE** pPtr);

    int  SS_ReleaseStructs();
   
    int  GAS_Evolve(GA_STRATEGY* pStrategy);   
    int  GAS_LoadFromFile(CString filePath, GA_STRATEGY* pStrategy);
    void GAS_SaveToFile(CString filePath, GA_STRATEGY* pStrategy, GAPopulation* pop);
	void GAS_SaveToFile(CString filePath, GA_STRATEGY* pStrategy);

	void GAS_presort(GAPopulation* pop);
	
    int  GAS_ExecuteStrategy(GA_STRATEGY* pStrategy, GA_PROTOCOL_STATS* pProtocolStats = NULL);
    int  GAS_ExecuteProtocol(GA_STRATEGY* pStrategy, int* pNumMessages = NULL);    
    int  GAS_EvaluateSuccess(GA_STRATEGY* pStrategy, BOOL* bSecure);    
    
    //int  GAS_ExecuteProtocolRecursive(GA_STRATEGY* pStrategy, int actualNodeCount, BOOL* bSecure);
    int  GAS_ExecuteNodesOriented(GA_STRATEGY* pStrategy, SENSOR_NODE* pCentralNode, int* pNumMessages = NULL);
    int  GAS_ExecuteGroupOriented(GA_STRATEGY* pStrategy, SENSOR_NODE* pCentralNode, int* pNumMessages = NULL);
    
    int GAS_SetLinkKeys(GA_STRATEGY* pStrategy);
    static int GAS_AllocateDefaultStrategyStructs(GA_STRATEGY* pStrategy);
	static int GAS_ClearDefaultStrategyStructs(GA_STRATEGY* pStrategy);
	static int GAS_CopyStrategyStructs(GA_STRATEGY* pOriginal, GA_STRATEGY* pCopy);
    static int GAGS_GetClosestDistNeigh(SENSOR_NODE* pCentralNode, SENSOR_NODE* pNeighNode, SENSOR_NODE** ppClosestSensor, int* pClosestSensorIndex, GAS_ELEMENT_TYPE relativeNodeIdentif, DWORD protocolTypeFlags);
    
    int  GAS_PrepareMutualWhisperStrategy(GA_STRATEGY* pStrategy);
    int  GAS_PrepareDanAmplifStrategy(GA_STRATEGY* pStrategy);
    int  GAS_PrepareRossAmplifStrategy(GA_STRATEGY* pStrategy);

    int  GAS_PruneProtocol(GAS_ELEMENT_TYPE* orig, GAS_ELEMENT_TYPE* pruned, DWORD maxInstruct, float origFitness, int pruneRepeat, float* insFitnessGain, DWORD sfncSlots);       
    int  GAS_ParseProtocol(CString textProtocol, GAS_ELEMENT_TYPE* protocol, DWORD* maxInstruct, DWORD* maxParties, GA_STRATEGY* pStrategy = NULL);
    int  GAS_ParseRelativeNodeIdentif(CString value, GAS_ELEMENT_TYPE* pRelIdentif);
    static int GAS_ValidateInstructionCorrectness(GAS_ELEMENT_TYPE* ins, GA_STRATEGY* pStrategy, DWORD flags = GAS_FLAG_ALL);
	static void GAS_CanonizeInstruction(GAS_ELEMENT_TYPE* ins, GA_STRATEGY* pStrategy);
    int  GAS_UncompromisedKeyShared(GA_STRATEGY* pStrategy, SENSOR_NODE* pNode1, SENSOR_NODE* pNode2);   
    int  GAS_GetAllSimpleComponents(GA_STRATEGY* pStrategy, int compositeID, lii* pSimpleList);
    
    static void GAS_initializer_random(GAGenome &g);
    static void GAGS_initializer_random(GAGenome &g);
	static void GA_initializer_load(GAGenome &g, DWORD protocolTypeFlags);
    static void GAS_initializer_load(GAGenome &g);
    static void GAGS_initializer_load(GAGenome &g);
    static float GAS_fitness(GAGenome &g);
    static int GAS_mutator(GAGenome &g, float pmut);
    static int GAGS_mutator(GAGenome &g, float pmut);
    static int GAS_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2);
    static int GAGS_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2);


    int  GABP_Evolve(GA_STRATEGY* pStrategy, BOOL bBlackPattern);   
    int  GABWP_Evolve(GA_STRATEGY* pWhiteStrategy, GA_STRATEGY* pBlackStrategy);   
    static void GABP_initializer_random(GAGenome &g);
    static float GABP_fitness(GAGenome &g);
    static float GAWP_fitness(GAGenome &g);
    static float GADP_fitness(GAGenome &g, BOOL bBlack, BOOL bWhite);
    static int GABP_mutator(GAGenome &g, float pmut);
    static int GABP_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2);
    static int GABP_sort(GA1DArrayGenome<int>* pGenom);
    static int GABP_FormatAsciiPattern(GAGenome* pG, CString* pValue, BOOL bToScreen); 
    static int GABP_FormatAsciiPattern(NODES_DEPLOY_PATTERN* pDeployPattern, CString* pValue, BOOL bToScreen); 

    static int GADP_SavePopulation(CString fileName, GASteadyStateGA* pGA, GA1DArrayGenome<int>* pGenomeTemp);
    static int GADP_ParsePattern(CString pattern, NODES_DEPLOY_PATTERN* pPattern);

//    static int GA_GetBestGenom(GA_STRATEGY* pStrategy, GA1DArrayGenome<int>* pBestGenom, GA1DArrayGenome<int>* pGenomeTemp);
    
    
    int GetRandomNodes(SENSOR_NODE** ppNodesList, DWORD numNodes);  // selects randomly numNodes nodes from white sensor list
    int GetNodesByID(SENSOR_NODE** ppNodesList, DWORD numNodes, lii* pIDList);  // selects numNodes nodes from white sensor list according to givenm ID list

    static int EG_SharedKeysProb(DWORD poolSize, DWORD ringSize, DWORD numSharedKeys, double* pProb);
    static int EG_NodeCaptureResilience(DWORD poolSize, DWORD ringSize, DWORD numComprNodes, double* pFraction);
	int CalcKeyChainUse(BASIC_INIT_DATA* pBasicSettings, float* avg, int* min, int* max);
    
    
    
    
    int NCR_Evolve(GA_STRATEGY* pStrategy);
    static float NCR_fitness(GAGenome &g);
    static void NCR_initializer_random(GAGenome &g);
    static void NCR_initializer_best(GAGenome &g);
    static int NCR_mutator(GAGenome &g, float pmut);
    static int NCR_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2);
    static float NCR_PickAlg(int type);
    static float NCR_PickAlg2(int type);
    static float NCR_PickAlg3(int type, int numRandomlyCaptured = 0);
	int NCR_compute(BASIC_INIT_DATA* pBasicSettings, int numCaptured, DWORD *pSecure, DWORD *pCompr);

    // 
    //  BATCH TESTS METHODS SUITABLE FOR WORKER THREADS 
    //
    static UINT ExecuteFixedTest(LPVOID pParam);
    static UINT ExecuteVariableRangeTest(LPVOID pParam);
    static UINT ExecuteVariableBWRatioTest(LPVOID pParam);
    static UINT ExecuteVariableBlackReceiveRatioTest(LPVOID pParam);
    static UINT ExecuteVariableTotalBSNumberTest(LPVOID pParam);
    static UINT ExecuteVariableActiveBSNumberTest(LPVOID pParam);
    static UINT ExecuteVariableComprNodesTest(LPVOID pParam);
    static UINT ExecuteVariableWhiteNodesTest(LPVOID pParam);
    static UINT ExecuteVariableDropCheckProbTest(LPVOID pParam);
    static UINT ExecuteVariableComprFractTest(LPVOID pParam);
    static UINT ExecuteGABPEvolve(LPVOID pParam);
    static UINT ExecuteEAProtocolSearchTest(LPVOID pParam);
    static UINT ExecuteEABlackPatternSearchTest(LPVOID pParam);
    static UINT ExecuteEAWhitePatternSearchTest(LPVOID pParam);
    static UINT ExecuteEABlackWhitePatternSearchTest(LPVOID pParam);
    static UINT ExecuteEvolveCircuitTest(LPVOID pParam);
    static UINT ExecuteEvolveNodePickTest(LPVOID pParam);
    
    static UINT ExecuteVariableTest(TEST_THREAD_PARAM* pThreadParam, float* pFloatVariable = NULL, DWORD* pDWORDVariable = NULL, DWORD flags = 0);

    int KeyPredistrib_EGScheme(BASIC_INIT_DATA* pBasicSettings);
    int KeyPredistrib_ClearKeys(BASIC_INIT_DATA* pBasicSettings);

    int PrintBasicDeployInfo();

    //ROUTING
	//JK
	int Routing_Test(BASIC_INIT_DATA* pBasicSettings);
	int GenerateMessages(BASIC_INIT_DATA* pBasicSettings, int* data);
	static UINT RunGARoute(LPVOID pParam);
	static int RunGARouteEvol(BASIC_INIT_DATA* pBasicSettings);
	

protected:
    static BOOL HasKeyDistribStatus(DWORD keyFlags);
    static int Dir_Create(CString path);
    static int DiscoverNeighbours(BASIC_INIT_DATA* pBasicSettings, DWORD discoverFlags);
    static int DiscoverNeighbours(NEIGH_LIST_NODE* first, NEIGH_LIST_NODE* second, DWORD discoverFlags);
    static int DiscoverNeighbours(NEIGH_LIST_NODE* pList, DWORD discoverFlags);
    static int DiscoverNeighbours(SENSOR_NODE* pSensor, SENSOR_NODE* pSensor2, DWORD discoverFlags);

    int ExecuteTests(BASIC_INIT_DATA* pBasicSettings, DWORD flagsWhatSave, CString basicPath, DWORD numSuffix = 1);

    int CalculateBasicDeployInfoString(BASIC_INIT_DATA* pBasicSettings, CString* pInfo);

    static int Array_Add(INT_DATA_BLOB* pArray, INT_DATA_BLOB* pPlusArray);
    static int Array_Add(INT_DATA_BLOB* pArray, INT_DATA_BLOB* pPlusArray, int plusStartOffset, int plusEndOffset);
    static int Array_LoadFromFile(CString filePath, INT_DATA_BLOB* pBlob);

    static DWORD Fact(DWORD value);
};

#endif