
#include "stdafx.h"
#include "TestQueue.h"
#include "SensorSimCore.h"
#include "SensorSimDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSensorSimCore*  pSensorSim;
extern CSensorSimDlg*  pSensorSimDlg;

CTestQueue::CTestQueue()
{
    m_testQueueExit = FALSE;
}

int CTestQueue::ExecuteTestThread(BASIC_INIT_DATA* pBasicSettings, DWORD testType) {
    int                     status = STAT_OK;
    TEST_THREAD_PARAM*      pThreadParam = new TEST_THREAD_PARAM;
    CString                 cfgScriptPath;
    AFX_THREADPROC          threadProc = NULL;
    
    // SET BASIC SETTINGS
    pThreadParam->basicSettings = *pBasicSettings;
    
    LOG_INSERTSEPARATOR;
    
    // INIT RANDOM GENERATOR
    pSensorSim->m_sensorEnv.InitRandomGenerator(&(pThreadParam->basicSettings));
    
    // SET SEED FIXED FOR FUTURE RE-EXECUTIONS OF SAME TEST
    pThreadParam->basicSettings.rndGen.useFixedSeed = TRUE;

    // SET TEST TYPE
    pThreadParam->basicSettings.testType = testType;

    // SAVE CONFIGURATION SCRIPT
    cfgScriptPath.Format("%s\\!cfgScript.ini", pThreadParam->basicSettings.batchTest.outputPath);
    status = pSensorSim->m_sensorEnv.SaveConfigScript(cfgScriptPath, &(pThreadParam->basicSettings));

    // TODO: CHECK FOR ALREADY RUNNING THREADS AND END THEM CORRECTLY
    // INTERRUPT PREVIOUS THREAD
    if (pSensorSim->m_pActualThread != NULL) TerminateThread(pSensorSim->m_pActualThread->m_hThread, 0);

    // RESET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = FALSE;

    switch (testType) {
        case FLAG_TESTTYPE_FIXEDTRANSMRANGE: threadProc = CSensorEnv::ExecuteFixedTest; break;
        case FLAG_TESTTYPE_VARIABLETRANSMRANGE: threadProc = CSensorEnv::ExecuteVariableRangeTest; break;
        case FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO: threadProc = CSensorEnv::ExecuteVariableBWRatioTest; break;
        case FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY: threadProc = CSensorEnv::ExecuteVariableBlackReceiveRatioTest; break;
        case FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT: threadProc = CSensorEnv::ExecuteVariableTotalBSNumberTest; break;
        case FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT: threadProc = CSensorEnv::ExecuteVariableActiveBSNumberTest; break;
        case FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT: threadProc = CSensorEnv::ExecuteVariableComprNodesTest; break;
        case FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT: threadProc = CSensorEnv::ExecuteVariableWhiteNodesTest; break;
        case FLAG_TESTTYPE_VARIABLEDROPCHECKPROB: threadProc = CSensorEnv::ExecuteVariableDropCheckProbTest; break; 
        case FLAG_TESTTYPE_VARIABLECOMPRNODES: threadProc = CSensorEnv::ExecuteVariableComprFractTest; break; 
        default: status = STAT_USERDATA_BAD;
    }

    if (threadProc != NULL) {
        // RUN WORKER THREAD FOR EXECUTION
        pSensorSim->m_pActualThread = AfxBeginThread(threadProc, pThreadParam, THREAD_PRIORITY_LOWEST); 
    }

    return status;
}

int CTestQueue::Add(BASIC_INIT_DATA* pBasicSettings) {
    int              status = STAT_OK;
    CString          message;

    if (pBasicSettings != NULL) {
        // SET APP VERSION IF NOT SET BEFORE
        if (pBasicSettings->simulSWVersion == "" && pSensorSimDlg != NULL) pSensorSimDlg->GetBasicSettingsHeader(pBasicSettings);

        // ADD TO QUEUE
        m_testQueue.push_back(pBasicSettings);

        message.Format("Test number %d added into queue ('%s')", m_testQueue.size(),  pBasicSettings->batchTest.outputPath);
        LOG_INSERTSTRING(message);
    }

    return status;
}

UINT TestQueueRun(LPVOID pParam) {
    UINT                status = STAT_OK;
    TEST_THREAD_PARAM*  pThreadParam = (TEST_THREAD_PARAM*) pParam;
    bsl::iterator       iter;
    bsl*                pTestQueue = (bsl*) pThreadParam->pTestQueue;
    BASIC_INIT_DATA*    pBasicSettings;
    CString             cfgScriptPath;
    CString             message;
    
    // EXECUTE TESTS IN QUEUE
    for (iter = pTestQueue->begin(); iter != pTestQueue->end(); iter++) {
        pBasicSettings = *iter;
        
        message.Format("Next test from queue ('%s')", pBasicSettings->batchTest.outputPath);
        LOG_INSERTSTRING(message);
        LOG_INSERTSEPARATOR;

        // INIT RANDOM GENERATOR
        pSensorSim->m_sensorEnv.InitRandomGenerator(pBasicSettings);
    
        // SET SEED FIXED FOR FUTURE RE-EXECUTIONS OF SAME TEST
        pBasicSettings->rndGen.useFixedSeed = TRUE;

        // SAVE CONFIGURATION SCRIPT
        cfgScriptPath.Format("%s\\!cfgScript.ini", pBasicSettings->batchTest.outputPath);
        status = pSensorSim->m_sensorEnv.SaveConfigScript(cfgScriptPath, pBasicSettings);

        status = pSensorSim->m_sensorEnv.ExecuteTestsBySettings(pBasicSettings);
    }

    // CLEAR LOCAL TEST QUEUE
    for (iter = pTestQueue->begin(); iter != pTestQueue->end(); iter++) {
        pBasicSettings = *iter;
        if (pBasicSettings != NULL) delete pBasicSettings;
    }
    pTestQueue->clear();

    LOG_INSERTSTRING("TestQueueRun finished");

    // EXIT APP IF REQUIRED
    if (pThreadParam->bExitApp && (pSensorSim->m_numRepetitionsElapsed >= pSensorSim->m_numRepetitions - 1)) {
		pSensorSim->PostMessage(WM_CLOSE);
	}
	
    delete (bsl*) pThreadParam->pTestQueue;
    delete pThreadParam;

    return status;
}

int CTestQueue::Run() {
    int                 status = STAT_OK;
    TEST_THREAD_PARAM*  pThreadParam = new TEST_THREAD_PARAM;
    bsl*                pLocalQueue;
    bsl::iterator       iter;
    BASIC_INIT_DATA*    pBasicSettings;
    BASIC_INIT_DATA*    pBasicSettings2;

    pLocalQueue = new bsl;

    // CREATE LOCAL COPY OF TEST QUEUE
    for (iter = m_testQueue.begin(); iter != m_testQueue.end(); iter++) {
        pBasicSettings = *iter;
        
        pBasicSettings2 = new BASIC_INIT_DATA;
        *pBasicSettings2 = *pBasicSettings;

        pLocalQueue->push_back(pBasicSettings2);
    }

    // SET LOCAL QUEUE AS THREAD PARAM
    pThreadParam->pTestQueue = pLocalQueue;
    pThreadParam->bExitApp = m_testQueueExit;

    // TODO: CHECK FOR ALREADY RUNNING THREADS AND END THEM CORRECTLY
    // INTERRUPT PREVIOUS THREAD
    if (pSensorSim->m_pActualThread != NULL) TerminateThread(pSensorSim->m_pActualThread->m_hThread, 0);

    // RESET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = FALSE;
    // RUN WORKER THREAD FOR QUEUE TESTS
    pSensorSim->m_pActualThread = AfxBeginThread(TestQueueRun, pThreadParam, THREAD_PRIORITY_LOWEST); 

    // CLEAR ORIGINAL TEST QUEUE
    Clear();

    return status;
}

int CTestQueue::Clear() {
    int                 status = STAT_OK;
    bsl::iterator       iter;
    BASIC_INIT_DATA*    pBasicSettings;
    
    for (iter = m_testQueue.begin(); iter != m_testQueue.end(); iter++) {
        pBasicSettings = *iter;

		// @TODO: check for memory leaks
        //if (pBasicSettings != NULL) delete pBasicSettings;
    }
    
    m_testQueue.clear();

    LOG_INSERTSTRING("Test queue cleared.");

    return status;
}

int CTestQueue::ExitAfterTests(BOOL bExit) {
    m_testQueueExit = bExit;
    return STAT_OK;
}
